@@ -193,7 +193,7 @@ sub set_serial {
 	}
 }
 
-sub set_virtconsole {
+sub set_console {
 	$self->virtconsole(bool(shift));
 }
 
@@ -352,6 +352,10 @@ sub monitor {
 	return $self->socket('monitor');
 }
 
+sub qmp {
+	return $self->socket('qmp');
+}
+
 sub serial {
 	my $num = shift;
 	my $serialport = $self->serialport;
@@ -364,7 +368,7 @@ sub serial {
 sub console {
 	my $num = shift // 0;
 	my $path = $self->socket_path("console-$num");
-	die "console only available with virtio=yes and console=yes\n"
+	die "console only available with console=yes\n"
 		unless $self->virtconsole || -e $self->socket_path('console-0');
 	return $self->socket("console-$num");
 }
@@ -423,6 +427,19 @@ sub devices_write {
 		default => 'on',
 	);
 
+	$self->devices_stanza($fh, chardev => 'qmp',
+		backend => 'socket',
+		server => 'on',
+		wait => 'off',
+		path => "$rundir/$name.qmp",
+	);
+	
+	$self->devices_stanza($fh, mon => 'qmp', 
+		mode => 'control',
+		chardev => 'qmp',
+		pretty => 'on',
+	);
+
 	if(defined $self->serialport) {
 		for(my $i = 0; $i < 4; $i++) {
 			$self->devices_stanza($fh, chardev => "serial-$i",
@@ -469,7 +486,7 @@ sub devices_write {
 			@vhost_net,
 		);
 		$self->devices_stanza($fh, device => undef, 
-			driver => 'virtio-net',
+			driver => $nictype,
 			netdev => "net-$i",
 			mac => $mac,
 		);
@@ -503,6 +520,14 @@ sub devices_write {
 			drive => "blk-$i",
 		);
 	}
+
+#	$self->devices_stanza($fh, drive => 'cdrom',
+#		if => 'none',
+#	);
+#	$self->devices_stanza($fh, device => 'ide0-cd1',
+#		driver => 'ide-cd',
+#		drive => 'cdrom',
+#	);
 }
 
 sub devices_file {
